import * as THREE from '../build/three.module.js';
import Stats from './jsm/libs/stats.module.js';

class App {
    constructor() {
        this._initialize();
    }

    _initialize() {
        const domWebGL = document.createElement('div');
        document.body.appendChild(domWebGL);
        this._domWebGL = domWebGL;

        const renderer = new THREE.WebGLRenderer({ antialias: true });
        renderer.shadowMap.enabled = true;
        renderer.shadowMap.type = THREE.PCFShadowMap;
        domWebGL.appendChild(renderer.domElement);
        this._renderer = renderer;

        const scene = new THREE.Scene();
        scene.background = new THREE.Color(0xf0f0f0);
        this._scene = scene;

        this._clock = new THREE.Clock();

        this._setupCamera();
        this._setupHelper();
        this._setupModel();
        this._setupAnimation();
        this._setupGUI();
        this._setupLight();
        
        window.onresize = this.resize.bind(this);

        this.resize();
    }

    _setupCamera() {
        const camera = new THREE.PerspectiveCamera(40, window.innerWidth / window.innerHeight, 1, 1000);
        camera.position.set(25, 25, 50);
        camera.lookAt(this._scene.position);

        this._camera = camera;
    }

    _setupHelper() {
        const axesHelper = new THREE.AxesHelper(10);
        this._scene.add(axesHelper);
    }

    _setupModel() {
        const geometry = new THREE.BoxGeometry(5, 5, 5);
        const material = new THREE.MeshBasicMaterial({ color: 0xffffff, transparent: true });
        const mesh = new THREE.Mesh(geometry, material);
        this._scene.add(mesh);

        this._mesh = mesh;
    }

    _setupAnimation() {
        const positionKF = new THREE.VectorKeyframeTrack(".position", 
            [0, 1, 2],
            [
                0, 0, 0, 
                30, 0, 0,
                0, 0, 0
            ]
        );
        const scaleKF = new THREE.VectorKeyframeTrack(".scale", 
            [0, 1, 2],
            [
                1, 1, 1,
                2, 2, 2,
                1, 1, 1
            ]
        );

        const xAxis = new THREE.Vector3(1, 0, 0);
        const qInitial = new THREE.Quaternion().setFromAxisAngle(xAxis, 0);
        const qFinal = new THREE.Quaternion().setFromAxisAngle(xAxis, Math.PI);
        const quaternionKF = new THREE.QuaternionKeyframeTrack(".quaternion", 
            [0, 1, 2], 
            [
                qInitial.x, qInitial.y, qInitial.z, qInitial.w, 
                qFinal.x, qFinal.y, qFinal.z, qFinal.w,
                qInitial.x, qInitial.y, qInitial.z, qInitial.w
            ]
        );
        const colorKF = new THREE.ColorKeyframeTrack(".material.color", 
            [0, 1, 2],
            [
                1, 0, 0,
                0, 1, 0,
                0, 0, 1
            ], 
            THREE.InterpolateDiscrete
        );
        const oppacityKF = new THREE.NumberKeyframeTrack(".material.opacity", 
            [0, 1, 2],
            [1, 0, 1]
        );
        const clip = new THREE.AnimationClip("default", 3, [scaleKF, positionKF, quaternionKF, colorKF, oppacityKF]);
        const mixer = new THREE.AnimationMixer(this._mesh);
        const clipAction = mixer.clipAction(clip);
        
        clipAction.play();

        this._mixer = mixer;
    }

    _setupGUI() {
        const stats = new Stats();
        this._domWebGL.appendChild(stats.dom);

        this._stats = stats;
    }

    _setupLight() {
        const light = new THREE.DirectionalLight(0xffffff, 2);
        light.position.set(1, 1, 1);
        this._scene.add(light);
    }

    update() {
        const delta = this._clock.getDelta();
        if(this._mixer) {
            this._mixer.update(delta);
        }
        this._stats.update();
    }

    render() {
        requestAnimationFrame(this.render.bind(this));
        
        this.update();
        this._renderer.render(this._scene, this._camera);
    }

    resize() {
        const aspect = window.innerWidth / window.innerHeight;

        this._camera.aspect = aspect;
        this._camera.updateProjectionMatrix();

        this._renderer.setPixelRatio( window.devicePixelRatio );
        this._renderer.setSize(window.innerWidth, window.innerHeight);
    }
}

window.onload = function () {
    (new App()).render();
}